/* src/main.c - Fichier principal
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: main.c,v 1.55 2003/12/27 16:19:26 romexzf Exp $
 */

#include <unistd.h>
#include <signal.h>
#include "main.h"
#include "config.h"
#include "debug.h"
#include "hash.h"
#include "outils.h"
#include "serveur.h"
#include "fichiers.h"
#include "chanserv.h"
#include "admin_user.h"
#include "nickserv.h"
#include "admin_cmds.h"
#include "admin_chan.h"
#include "admin_manage.h"
#include "divers.h"
#include "moduser.h"
#include "chanopt.h"
#include "opdeop.h"
#include "chaninfo.h"
#include "showcommands.h"
#include "aide.h"
#include "cs_register.h"
#include "timers.h"
#include "socket.h"
#include "ban.h"
#include "flood.h"
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef HAVE_VOTE
#include "vote.h"
#endif
#ifdef USE_MEMOSERV
#include "memoserv.h"
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif
#ifdef TESTBOT
#include "web2cs.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif

int ConfFlag = 0;
int lastresetparse = 0;
int lastcheckaccounts = 0;
int want_to_quit = 0;

/* Les needarg sonts les nombre d'arg minimum requis */
struct cs_commands cs_cmds[] = {
 /* cmd,            function,             niveau, needarg, flag, secure, 0*/
 /* Admins*/
 {  "die",          die,                  "", 4,      0, 2, 0, 0, "die" },
 {  "restart",      restart_bot,          "", 4,      0, 2, 0, 0, "restart" },
 {  "rehash",       rehash_conf,          "", 4,      0, 2, 0, 0, "rehash" },
 {  "adminlvl",     admin_level,          "", 4,      2, 2, 0, 0, "adminlvl" },
 {  "chcomname",    chcomname,            "", 4,      2, 2, 0, 0, "chcomname" },
 {  "chlevel",      chlevel,              "", 4,      2, 2, 0, 0, "chlevel" },
 {  "disable",      disable_cmd,          "", 4,      2, 2, 0, 0, "disable" },
#ifdef USE_WELCOMESERV
 {  "setglobwelcome",set_welcome,         "", 3,      0, 2, 0, 0, "setglobwelcome" },
#endif
 {  "inviteme",     inviteme,             "", 2,      0, 2, 0, 0, "inviteme" },
 {  "say",          admin_say,            "", 2,      2, 3, 0, 0, "say" },
 {  "write",        write_files,          "", 2,      0, 2, 0, 0, "write" },
 {  "cswall",       cs_wall,              "", 2,      1, 2, 0, 0, "cswall" },
 {  "global",       globals_cmds,         "", 2,      2, 2, 0, 0, "global" },
 {  "chan",         admin_chan,           "", 2,      1, 2, 0, 0, "chan" },
 {  "help",         aide,                 "", 0,      0, 0, 0, 0, "help" },
#ifdef HAVE_TRACK
 {  "track",         cmd_track,           "", 3,      1, 2, 0, 0, "track" },
#endif
 {  "user",         admin_user,           "", 2,      2, 2, 0, 0, "user" },
 {  "whoison",      whoison,              "", 2,      0, 3, 0, 0, "whoison" },
#ifdef HAVE_VOTE
 {  "vote",         do_vote,              "", 3,      0, 2, 0, 0, "vote" },
#endif
 /* Users (check with chan access)*/
 {  "unreg",        unreg_chan,           "", OWNERLEVEL, 1, 1, 0, 0, "unreg" },
 {  "renchan",      ren_chan,             "", OWNERLEVEL, 2, 1, 0, 0, "renchan" },
 {  "adduser",      add_user,             "", 450,    3, 1, 0, 0, "adduser" },
 {  "deluser",      del_user,             "", 450,    2, 1, 0, 0, "deluser" },
 {  "opall",        opall,                "", 400,    1, 1, 0, 0, "opall" },
 {  "deopall",      deopall,              "", 400,    1, 1, 0, 0, "deopall" },
 {  "clearmodes",   clearmodes,           "", 400,    0, 1, 0, 0, "clearmodes" },
 {  "voiceall",     voiceall,             "", 300,    1, 1, 0, 0, "voiceall" },
 {  "devoiceall",   devoiceall,           "", 300,    1, 1, 0, 0, "devoiceall" },
 {  "ban",          ban_cmd,              "", 100,    2, 1, 0, 0, "ban" },
 {  "unban",        unban,                "", 300,    2, 1, 0, 0, "unban" },
 {  "kickban",      kickban,              "", 300,    2, 1, 0, 0, "kickban" },
 {  "rdefmodes",    rdefmodes,            "", 300,    1, 1, 0, 0, "rdefmodes" },
 {  "rdeftopic",    rdeftopic,            "", 300,    1, 1, 0, 0, "rdeftopic" },
 {  "clearbans",    clear_bans,           "", 300,    0, 1, 0, 0, "clearbans" },
 {  "info",         info,                 "", 100,    0, 1, 0, 0, "info" },
 {  "invite",       invite,               "", 100,    0, 1, 0, 0, "invite" },
 {  "op",           op,                   "", 100,    1, 1, 0, 0, "op" },
 {  "deop",         deop,                 "", 100,    1, 1, 0, 0, "deop" },
 {  "kick",         kick,                 "", 100,    2, 1, 0, 0, "kick" },
 {  "mode",         mode,                 "", 100,    2, 1, 0, 0, "mode" },
 {  "topic",        topic,                "", 100,    2, 1, 0, 0, "topic" },
 {  "do",           admin_do,             "", 100,    2, 1, 0, 0, "do" },
 {  "devoice",      devoice,              "", 50,     1, 1, 0, 0, "devoice" },
 {  "voice",        voice,                "", 50,     1, 1, 0, 0, "voice" },
 {  "alist",        see_alist,            "", 1,      1, 1, 0, 0, "alist" },
 {  "unbanme",      unbanme,              "", 1,      1, 1, 0, 0, "unbanme" },
 /* Users (check with nick access)*/
 {  "listbadchans", list_badchans,        "", 1,      0, 0, 0, 0, "listbadchans" },
 {  "myaccess",     myaccess,             "", 1,      0, 0, 0, 0, "myaccess" },
 {  "drop",         drop_user,            "", 1,      1, 0, 2, 0, "drop" },
 {  "listbadnicks", list_badnicks,        "", 1,      0, 0, 0, 0, "listbadnicks" },
#ifdef HAVE_VOTE
 {  "voter",        voter,                "", 1,      0, 0, 2, 0, "voter" },
 {  "results",      vote_results,         "", 1,      0, 0, 0, 0, "results" },
#endif
#ifdef USE_MEMOSERV
 {  "memos",        memos,                "", 1,      2, 0, 1, 0, "memos" },
#endif
 {  "verify",       verify,               "", 1,      1, 0, 0, 0, "verify" },
 {  "register",     register_user,        "", 0,      2, 0, 1, 0, "register" },
 {  "login",        ns_login,             "", 0,      2, 0, 1, 0, "login" },
#ifdef USE_NICKSERV
 {  "recover",      recover,              "", 0,      0, 0, 1, 0, "recover" },
#endif
 {  "deauth",       deauth,               "", 0,      0, 4, 0, 0, "deauth" },
 {  "seen",         lastseen,             "", 0,      1, 0, 0, 0, "seen" },
 {  "regchan",      register_chan,        "", 0,      2, 0, 0, 0, "regchan" },
 {  "access",       show_access,          "", 0,      2, 1, 0, 0, "access" },
 {  "aide",         aide,                 "", 0,      0, 0, 0, 0, "aide" },
 {  "showcommands", showcommands,         "", 0,      0, 0, 0, 0, "showcommands" },
 {  "uptime",       uptime,               "", 0,      0, 0, 0, 0, "uptime" },
 {  "banlist",      banlist,              "", 0,      1, 1, 0, 0, "banlist" },
 {  "admin",        show_admins,          "", 0,      0, 0, 0, 0, "admin" },
 {  "whois",        cs_whois,             "", 2,      1, 2, 0, 0, "whois" },
 {  "ignorelist",   show_ignores,         "", 0,      0, 0, 0, 0, "ignorelist" },
 {  "oubli",        oubli_pass,           "", 0,      2, 0, 0, 0, "oubli" },
 {  "\001ping",     ctcp_ping,            "", 0,      0, 0, 0, 0, "\1ping" },
 {  "\001version\001", ctcp_version,      "", 0,      0, 0, 0, 0, "\1version\1" },
 {  "chaninfo",     chaninfo,             "", 0,      1, 1, 0, 0, "chaninfo" },
 /*moduser*/
 {  "moduser",      generic_moduser,      "", 450,    3, 1, 0, 0, "moduser" },
 {  "autoop",       moduser_autoop,       "", 450,    1, 1, 0, 0, "autoop" },
 {  "autovoice",    moduser_autovoice,    "", 450,     1, 1, 0, 0, "autovoice" },
 {  "protect",      moduser_protect,      "", 450,    1, 1, 0, 0, "protect" },
 /*chanopt*/
 {  "locktopic",    locktopic,            "", 400,    0, 1, 0, 0, "locktopic" },
 {  "strictop",     strictop,             "", 450,    0, 1, 0, 0, "strictop" },
 {  "nobans",       nobans,               "", 450,    0, 1, 0, 0, "nobans" },
 {  "noops",        noops,                "", 450,    0, 1, 0, 0, "noops" },
 {  "deftopic",     deftopic,             "", 400,    1, 1, 0, 0, "deftopic" },
 {  "defmodes",     defmodes,             "", 400,    1, 1, 0, 0, "defmodes" },
 {  "description",  description,          "", 450,    1, 1, 0, 0, "description" },
 {  "welcome",      csetwelcome,          "", 450,    1, 1, 0, 0, "welcome" },
 {  "setwelcome",   activwelcome,         "", 450,    1, 1, 0, 0, "setwelcome" },
 {  "banlevel",     banlevel,             "", 400,    1, 1, 0, 0, "banlevel" },
 {  "bantype",      bantype,              "", 400,    1, 1, 0, 0, "bantype" },
 {  "chanurl",      chanurl,              "", 450,    1, 1, 0, 0, "chanurl" },
 {  "motd",         define_motd,          "", 450,    1, 1, 0, 0, "motd" },
 {  "chanopt",      generic_chanopt,      "", 450,    2, 1, 0, 0, "chanopt" },
/* NOUVELLES COMMANDES ICI !*/
 {  "showconfig",   showconfig,           "", 4,      0, 2, 0, 0, "showconfig" },
#ifdef USE_WELCOMESERV
 {  "addwelcome",   add_welcome,          "", 4,      2, 2, 0, 0, "addwelcome" },
 {  "delwelcome",   del_welcome,          "", 4,      1, 2, 0, 0, "delwelcome" },
 {  "listwelcome",  list_welcome,         "", 4,      0, 2, 0, 0, "listwelcome" },
 {  "adminmotd",    set_motds,            "", 4,      1, 2, 0, 0, "adminmotd" },
#endif
 {  "abad",   badmanage,                  "", 3,      2, 2, 0, 0, "abad" },
 {  "set",   user_set,                    "", 1,      1, 0, 0, 0, "set" },
};

int cs_cmds_size = sizeof cs_cmds / sizeof *cs_cmds;

struct badchans *badchead = NULL;
struct badnicks *badnhead = NULL;
struct ignore *ignorehead = NULL;
aKill *killhead = NULL;
Lang *DefaultLang = NULL;

struct robot bot;
#ifdef USE_WELCOMESERV
struct bots ws;
char user_motd[400];
char admin_motd[400];
#endif
struct bots cs;


aChan *chan_tab[CHANHASHSIZE] = {0}; 	/* hash chan*/
anUser *user_tab[USERHASHSIZE] = {0}; 	/* hash username*/
aNick **num_tab[MAXNUM] = {0}; 			/* table num -> struct nick*/
aServer *serv_tab[MAXNUM] = {0}; 		/* table numserv -> serv info*/
aNick *nick_tab[NICKHASHSIZE] = {0}; 	/* hash nick*/

/*            FONCTIONS COMMANDES
 *            =-=-=-=-=-=-=-=-=-=
 *
 *    Il y a deux types de fonctions commandes.
 *    Les normales :
 *       parv[0] = pseudo du robot appel
 *       parv[1->parc] = arguments de la commandes
 *    Et les commandes salons (voir strict cs_commands) :
 *       parv[0] = nom de la commande appele
 *       parv[1] = salon appel
 *       parv[2->parv] = arguments de la commande (aprs le nom du salon)
 *    Toutes les commandes doivent respecter ce schma.
 *    Lorsqu'une commande salon est tape sur un salon, et que le premier
 *    argument n'est pas un nom de salon, le nom du salon actuel est plac
 *    automatiquement comme premier argument.
 *
 *    Chaque commande emploie une entte particulire :
 *    int commande(aNick *, aChan *, int, char **)
 *      aNick *nick = structure nickinfo de l'user qui a appel la fonction
 *      aChan *chaninfo = structure chaninfo du salon si la cmd est chan=1 (sinon chaninfo=NULL)
 *      int parc = nombre d'arguments *SANS COMPTER PARV[0]*. (si le dernier arg
 *                 est parv[5], alors parc = 5.)
 *      char **parv = tableau contenant les arguments (une ligne = un mot).
 *
 *    Outils utiles :
 *      parv2msgn(int parc, char **parv, int position, char *buf, int size)
 *          permet de convertir une fin de tableau en un message normal.
 *          il faut spcifier le tableau et le nombre d'arguments contenu
 *          ainsi que la position  laquelle on va commencer  enregistrer
 *          le contenu dans la variable envoye en paramtre (taille max size).
 *		parv2msg(int parc, char **parv, int position, int size)
 *			idem mais renvoit un pointeur vers la string forme plutot que l'crire dans buf
 *      num2nickinfo(const char *num)
 *          renvoit la struct de la num spcifie (accs direct dsormais avec la table des nums)
 *      csntc(aNick *nick, register const char *format, ...)
 *          envoyer une notice avec le CS
 *    Pour plus d'outils, voir outils.c.
 */

void sig_handler (int c)
{
	Debug(W_WARN|W_MAX, "SIGTERM/SIGINT reu! -- abandon..");
	want_to_quit = 1;
	signal(SIGINT, &sig_handler);
	signal(SIGTERM, &sig_handler);
}

void sig_reload (int c)
{
	Debug(W_WARN, "Signal HUP reu!");
	signal(SIGHUP, &sig_reload);
}

void sig_alarm(int c)
{
	time_t current = time(NULL);

	bot.lasttime = current;		/* mise  jour des infos  */
	bot.lastbytes = bot.dataQ;	/* pour les stats traffic */

	check_kills();
	check_ignores();
	if((lastcheckaccounts + 7200) < current)
	{
		check_accounts();
		check_bans(1);
		lastcheckaccounts = current;
	}
	else check_bans(0);

#ifdef DEBUG
	if((lastresetparse + WAIT_DEL_PARSES) < current)
	{
		remove(LOG_PARSES);
		lastresetparse = current;
	}
#endif
	alarm(ALARMINTERVAL);
}

int main(int argc, char **argv)
{
	int r, restarted = 0, silent = 0, tmp = 0, opt, background = 1;
	FILE *fd;
	char buff[150] = {0};

	while((opt = getopt(argc, argv, "hn")) != EOF)
		switch(opt)
		{
			case 'n':
				background = 0;
				break;
			case 'h':
				silent = 1;
				break;
			default:
				printf("Syntaxe: %s [-hn]\n", argv[0]);
				exit(EXIT_FAILURE);
				break;
		}

	chdir(BINDIR); /* on se place dans le rpertoire principal de scoderz */

	if((fd = fopen(SCODERZ_PID, "r")))
	{
		if(fgets(buff, sizeof buff, fd) && sscanf(buff, "%d", &tmp) == 1)
		{
			fprintf(stderr, "SCoderZ est dj lanc sur le pid %d.\n", tmp);
			fprintf(stderr, "Si ce n'est pas le cas, supprimez le fichier '" SCODERZ_PID "' et recommencez.\n");
		}
		fclose(fd);
		exit(EXIT_FAILURE);
	}
	else bot.uptime = time(NULL);

	if(load_config(FICHIER_CONF) == -1)
	{
		Debug(W_TTY, "Erreur lors de la lecture de la configuration");
		exit(EXIT_FAILURE);
	}

#ifdef TESTBOT
	for(r = 0; r < LANGMSGNB;++r)
		if(!DefaultLang->msg[r])
		{
			Debug(W_TTY, "lang: langage par dfaut incomplet (msgid %d manquante)", r);
			exit(EXIT_FAILURE);
		}
#endif

	if((fd = fopen("uptime.tmp", "r")))
	{
		if(fgets(buff, sizeof buff, fd))
			sscanf(buff, "%lu %d", (unsigned long *) &bot.uptime, &lastresetparse);
		fclose(fd);
		remove("uptime.tmp");
		restarted = 1;
	}

	bot.sock = 0;
	r = load_cmds();
	if(!silent) printf("Chargement des commandes IRC... OK (%d)\n", r);
	BuildCommandsTable();
	r = load_chans();
	if(!silent) printf("Base de donne Salon charge (%d)\n", r);
	r = load_users();
	if(!silent) printf("Base de donne User charge (%d)\n", r);
	r = load_badchans();
	if(!silent) printf("Chargement des bads chans... OK (%d)\n", r);
#ifdef USE_WELCOMESERV
	load_welcome();
#endif
	r = load_badnicks();
	if(!silent) printf("Chargement des bads nicks... OK (%d)\n", r);
#ifdef HAVE_VOTE
	load_votes();
#endif

	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP, &sig_reload);
	signal(SIGINT, &sig_handler);
	signal(SIGTERM,&sig_handler);
	signal(SIGALRM, &sig_alarm);

	if(background && (r = fork()) == -1)
	{
		puts("Impossible de se lancer en background.");
		exit(EXIT_FAILURE);
	}

	if(!restarted && !silent && (!background || r)) puts("Services CoderZ " SPVERSION "  Inter System");
	if(GetConf(CF_PREMIERE) && !silent)
	{
		puts("Lorsque les services seront sur votre rseau IRC,");
		printf("tapez : /%s %s <username> <email> <pass>\n", cs.nick, RealCmd("register"));
	}
	if(background && r > 1)
	{
		if(!restarted && !silent) puts("Lancement en background...");
		exit(0);
	}

#ifdef SQLLOG
	bot.sql_id = NULL;
	if(!sql_init())
	{
		printf("log:SQL connexion  %s:%d (user=%s base=%s) impossible...\n",
					bot.sql_ip, bot.sql_port, bot.sql_user, bot.sql_db);
		exit(EXIT_FAILURE);
	}
#endif

#ifdef TESTBOT
	w2c_initsocket();
#endif

#ifdef N2K
    sprintf(cs.num, "%s%s", bot.servnum, "AAA");
#ifdef USE_WELCOMESERV
    sprintf(ws.num, "%s%s", bot.servnum, "AAB");
#endif

#else
    sprintf(cs.num, "%s%s", bot.servnum, "AA");
#ifdef USE_WELCOMESERV
    sprintf(ws.num, "%s%s", bot.servnum, "AB");
#endif

#endif

	if(!(fd = fopen(SCODERZ_PID, "w")) && !silent) puts("Impossible d'crire le fichier PID...");
	else
	{
		fprintf(fd, "%d\n", getpid());
		fclose(fd);
	}

	while(!want_to_quit)
	{
		unsigned long int irc_ip = resolve(bot.ip);

		if(irc_ip <= 0) putlog(LOG_ERREURS, "DNS lockup a chou pour [%s]", bot.ip);
		else if(init_bot(irc_ip) < 0) putlog(LOG_ERREURS, "Connexion choue.");
		else run_bot();

		if (!want_to_quit)
		{
			int i = 0;
			aChan *chan;
			putlog(LOG_ERREURS, "Dconnect du serveur.");

			purge_nickandserv();

			for(;i < CHANHASHSIZE;i++) for(chan = chan_tab[i];chan;chan = chan->next) DelCJoined(chan);
			sleep(WAIT_CONNECT);
		}
	}

	CleanUp();
	putlog(LOG_PARSES, "Fermeture normale du programme");
	return 0;
}
