/* src/memoserv.c - Diverses commandes sur le module memoserv
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: memoserv.c,v 1.28 2003/12/27 23:36:17 romexzf Exp $
 */

#include "main.h"
#ifdef USE_MEMOSERV
#include "memoserv.h"
#include "fichiers.h"
#include "aide.h"
#include "hash.h"
#include "outils.h"
#include "cs_cmds.h"
#include "add_info.h"
#include "del_info.h"

void show_notes(aNick *nick)
{
	aMemo *memo = nick->user->memohead;
	int i = 0;
	for(;memo;memo = memo->next) if(!memo->read) i++;

	if(i) csntc(nick, "Vous avez reu\2 %d\2 nouveau%s memo%s, pour le%s lire tapez \2/%s %s LIRE <ALL|N du memo>\2",
			i, PLURX(i), PLUR(i), PLUR(i), cs.nick, RealCmd("memos"));
	return;
}

int memos(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aMemo *memo, *mt = NULL;
	int i = 0, p = 0, found = 0;
	const char *cmd = parv[1];

	if(!strcasecmp(cmd, "LIRE") || !strcasecmp(cmd, "READ"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe : %s LIRE <NEW|ALL|N DU MEMO>", RealCmd("memos"));

		if(is_num(parv[2])) i = atoi(parv[2]);
		else if (!strcasecmp(parv[2], "ALL")) i = -1;
		else if (!strcasecmp(parv[2], "NEW")) i = -2;
		else return csntc(nick, "Syntaxe : %s LIRE <ALL|NEW|N DU MEMO>", RealCmd("memos"));

		for(memo=nick->user->memohead;memo;memo=memo->next)
		{
			p++;
			if(p == i || i == -1 || (i == -2 && !memo->read))
			{
				csntc(nick, "%c#%d De\2 %s\2 [%s] : %s", memo->read ? ' ' : '*', p, memo->de, get_time(memo->date), memo->message);
				found = memo->read = 1;
			}
		}
		if(found)
		{
			if(i == -1) csntc(nick, GetReply(nick, L_MEMOTOTAL), p, PLUR(p), cs.nick, RealCmd("memos"));
			else if(i == -2) csntc(nick, GetReply(nick, L_MEMONEWTOTAL), p, PLURX(p), PLUR(p));
			else csntc(nick, GetReply(nick, L_MEMOEND), i, cs.nick, RealCmd("memos"), i);
		}
		else if(i == -1) csntc(nick, GetReply(nick, L_NOMEMOFOUND));
		else if(i == -2) csntc(nick, GetReply(nick, L_MEMONEWNOTFOUND));
		else csntc(nick, GetReply(nick, L_MEMONOTFOUND), i);
	}
	else if(!strcasecmp(cmd, "SUPPRIMER") || !strcasecmp(cmd, "ERASE")
		|| !strcasecmp(cmd, "SUPPR") || !strcasecmp(cmd, "DEL"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe : %s SUPPR <ALL|N DU MEMO>", RealCmd("memos"));

		if(is_num(parv[2])) i = atoi(parv[2]);
		for(p = 1, memo = nick->user->memohead;memo;p++, mt = memo, memo = memo->next)
		{
			if(!i)
			{
				free(mt);
				found++;
			}
			else if(p == i)
			{
				if(mt) mt->next = memo->next;
				else nick->user->memohead = memo->next;
				free(memo);
				found = 1;
				break;
			}
		}
		if(!i && found)
		{
			free(mt);
			csntc(nick, GetReply(nick, L_MEMOSDEL), found);
			nick->user->memohead = NULL;
		}
		else if(!i && !found) csntc(nick, GetReply(nick, L_NOMEMOFOUND));
		else if(found) csntc(nick, GetReply(nick, L_MEMODEL), i);
		else csntc(nick, GetReply(nick, L_MEMONOTFOUND), i);
		write_anuser(nick->user);
	}
	else if(!strcasecmp(cmd, "SEND") || !strcasecmp(cmd, "ENVOYER"))
	{
		anUser *user;
		aNick *who;
		char *tmp;

		if(parc < 3)
			return csntc(nick, "Syntaxe : %s ENVOYER <username> <message>", RealCmd("memos"));

		if(!(user = getuserinfo(parv[2])))
			return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[2]);

		if(user == nick->user)
			return csntc(nick, GetReply(nick, L_CANTSENDYOURSELF));

		if(UNoMemo(user) && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_XNOMEMO), user->nick);

		tmp = parv2msg(parc, parv, 3, MEMOLEN + 3);
		if(*tmp == ':') tmp++;

		if(strlen(tmp) > MEMOLEN)
			return csntc(nick, GetReply(nick, L_MEMOLEN), MEMOLEN);

		if((who = user->n) && !(who->flag & N_AWAY)) csntc(who, GetReply(who, L_MEMOFROM), nick->nick, nick->user->nick, tmp);

		for(memo = user->memohead;memo;memo = memo->next) if(!strcasecmp(memo->de,nick->user->nick)) i++;
		if(i >= MAXMEMOS && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_MAXMEMO), user->nick, MAXMEMOS);

		add_memo(user, nick->user->nick, time(NULL), tmp, (who && !(who->flag & N_AWAY)) ? 1 : 0);
		csntc(nick, GetReply(nick, L_MEMOSENT), user->nick);
		csntc(nick, "\2Texte:\2 %s", tmp);
		write_anuser(user);
	}
	else
	{
		csntc(nick, "La sous commande\2 %s\2 n'existe pas.", cmd);
		syntax_cmd(nick, GetCmdIndexbyName("MEMOS"));
		return 0;
	}
	return 1;
}
#endif
