/* src/sql_log.c - sql log
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: sql_log.c,v 1.8 2003/12/27 16:19:27 romexzf Exp $
 */

#include "main.h"
#ifdef SQLLOG
#include "sql_log.h"
#include "debug.h"
#include "cs_cmds.h"
#include <mysql.h>
#include <errmsg.h>
#include <stdarg.h>

static const char table_dec[] = "0123456789";

int sql_query(const char *pattern, ...)
{
	char buf[1100], t;
 	register char *p = buf;
 	va_list vl;

	va_start(vl, pattern);
	while((t = *pattern++))/* %sa (t = %, *pattern = s)*/
 	{
 		if(t == '%')
 		{
 			t = *pattern++;/* on drop le formateur (t = s, *pattern = a)*/
 			if(t == 's')
 			{
 				register char *tmps = va_arg(vl, char *);
 				if(tmps)
 					while(*tmps)/* copie de la string*/
 					{
 						if(*tmps == '\\' || *tmps == '\'' || *tmps == '"' || *tmps == '%') *p++ = '\\';
 						*p++ = *tmps++;
 					}
 				continue;
 			}
 			if(t == 'c')
 			{
 				*p++ = (char) va_arg(vl, int);
 				continue;
 			}
 			if(t == 'T')
 			{
 				unsigned long tmpi = va_arg(vl, unsigned long);
 				int pos = 31;
 				char bufi[32];

 				if(tmpi == 0)
 				{
 					*p++ = '0';
 					continue;
 				}
 				while(tmpi)/* on converti une int en base 10 en string*/
 				{		/* criture dans l'ordre inverse 51 > '   1' > '  51'*/
 					bufi[pos--] = table_dec[tmpi % 10];
 					tmpi /= 10;
 				}
 				while(pos < 31) *p++ = bufi[++pos];
 				continue;
			}
 			if(t != '%')
 			{/* on sous traite (depuis le dbut..)  vsnprintf*/
 				vsprintf(p, pattern - 2, vl);/* on remet le %format*/
 				break;
 			}
 		}
 		*p++ = t;
	}
 	if(t == '\0') *p = '\0';
	va_end(vl);
	if(mysql_query(bot.sql_id, buf))/* erreur lors de la requete*/
 	{
		if(mysql_errno(bot.sql_id) == CR_SERVER_LOST) /* ok ce n'est qu'une dco*/
		{
			if(!sql_init()) /*essayons une reconnexion */
				Debug(W_WARN, "DB SQL inaccessible -- nouvel essai lors du prochain query.");
			else
			{
				cswallops("DB SQL perdue -- reconnexion OK -- envoi du query.");
				mysql_query(bot.sql_id, buf);
				return 1;
			}
		}
		Debug(W_WARN|W_MAX, "sql_query failed (%s)", buf);
		return -1;
	}
	return 1;
}

int sql_init(void)
{
	MYSQL_RES *result = NULL;
	int culog = 1, cclog = 1;

 	bot.sql_id = mysql_init(NULL);
 	if(!mysql_real_connect(bot.sql_id, bot.sql_ip, bot.sql_user, bot.sql_pass, bot.sql_db, bot.sql_port, NULL, 0))
 		return 0;

 	mysql_query(bot.sql_id, "SHOW TABLES;");/* on vrifie si les tables existent */
 	if((result = mysql_store_result(bot.sql_id)))
 	{
 		MYSQL_ROW row;

 		while((row = mysql_fetch_row(result)))
 		{
 			mysql_field_seek(result, 0);
 			if(!strcmp(row[0], "usercmdslog")) culog = 0;
 			else if(!strcmp(row[0], "chancmdslog")) cclog = 0;
 		}
 		mysql_free_result(result);
	}

 	if(culog)
 		if(sql_query("CREATE TABLE usercmdslog (\n user CHAR(%d),\n TS INT(4),\n cmd CHAR(15),\n log CHAR(200));",
 				NICKLEN + 1) < 0)
 					return 0;
 	if(cclog)
 		if(sql_query("CREATE TABLE chancmdslog (\n user CHAR(%d),\n chan CHAR(%d),\n TS INT(4),\n cmd CHAR(15),\n log CHAR(200));",
 				NICKLEN + 1, REGCHANLEN + 1) < 0)
 					return 0;
	return 1;
}
#endif
